@@ -1,4 +1,5 @@
 import json
+import os
 
 from pydantic.typing import NoneType
 
@@ -7,7 +8,7 @@ from .const import CSV_SIZE
 from typing import Dict
 import pytest
 
-from planchet.core import Job, COMPLETE, IN_PROGRESS, RECEIVED, SERVED
+from planchet.core import Job, COMPLETE, IN_PROGRESS, RECEIVED, SERVED, WRITE_ONLY, READ_ONLY
 
 
 @pytest.mark.parametrize('batch_size', [1, 2, 5, 10, 13, 30, 32])
@@ -61,7 +62,8 @@ def test_restore_job(reader, writer, ledger):
     value = json.dumps({
         'metadata': metadata,
         'reader_name': type(reader).__name__,
-        'writer_name': type(writer).__name__
+        'writer_name': type(writer).__name__,
+        'dumping': False
     })
     ledger.set(job_key, value)
     n_skips: int = 10
@@ -123,3 +125,40 @@ def test_stats(job):
     assert stats['served'] == n_served
     assert stats['received'] == 2
     assert stats['status'] == IN_PROGRESS
+
+
+def test_writing_job(writing_job, output_fp, ledger, csv_items):
+    writing_job.receive(csv_items, False)
+    assert os.path.exists(output_fp)
+    with open(output_fp) as fh:
+        for i, line in enumerate(fh, start=1):
+            assert len(line.split(',')) == 2
+    n_items = len(csv_items)
+    assert i == n_items
+    keys = ledger.scan_iter(f'{writing_job.name}:*')
+    assert len(list([k for k in keys])) == n_items
+
+
+def test_reading_job(reading_job):
+    n_items = 5
+    items = reading_job.serve(n_items)
+    assert items, items
+
+
+def test_mark_errors(job, ledger):
+    ids = [1, 2, 3, 4]
+    job.mark_errors(ids)
+    records = ledger.scan_iter(f'{job.name}:*')
+    items = [int(itm.decode('utf8')[-1]) for itm in records]
+    assert items == ids
+
+
+def test_mark_errors_received(job, ledger):
+    n_items = 10
+    items = job.serve(n_items)
+    ids = [id_ for id_, _ in items]
+    job.receive(items, False)
+    with pytest.raises(ValueError):
+        job.mark_errors(ids)
+    keys = ledger.scan_iter(f'{job.name}:*')
+    assert all([ledger.get(k).decode('utf8') == RECEIVED for k in keys])
